from pathlib import Path

from solar_registry.service.validator import ToolValidator


def test_validate_meta_json() -> None:
    workdir = str(
        (Path(__file__).parent / "testdata" / "stable_index_file_check").resolve()
    )
    validator = ToolValidator(workdir)

    validator.validate()



"""
测试工具模型定义

这里面的模型都是对外体现的
"""

from enum import Enum
from typing import Literal, Self

from pydantic import BaseModel, Field, ValidationInfo, model_validator

from .legacy import LegacySpec


class ParamChoice(BaseModel):
    value: str = Field(title="选项值")
    desc: str = Field(title="选项描述")


class ParamWidget(str, Enum):
    Code = "code"
    Text = "text"
    Number = "number"
    Choices = "choices"
    TextArea = "text-area"


class ParamDef(BaseModel):
    name: str = Field(
        title="参数名称",
        description="""
参数名称，在实际执行的时候会被转换为环境变量下发。
    
这些参数在用户 use 测试工具的时候会通过 with 进行设置，并最终会被传递给对应的脚本文件，传递的方式是 环境变量，映射方式如下：

TESTSOLAR_TTP_大写名称

举例:
- useVirtualEnv -> TESTSOLAR_TTP_USEVIRTUALENV
- workerCount -> TESTSOLAR_TTP_WORKERCOUNT

    """,
    )
    value: str = Field(title="参数值", description="格式必须是字符串")
    desc: str = Field("", title="参数描述信息")

    default: str = Field("", title="参数默认值")
    choices: list[ParamChoice] | None = Field(None, title="参数选项")

    # 兼容历史工具
    lang: str | None = Field(
        None, title="参数语言", description="用于前端UI显示，判断语法高亮类型"
    )
    input_widget: ParamWidget | None = Field(
        None,
        alias="inputWidget",
        title="输入控件类型",
        description="""
    输入控件的类型，用于前端界面显示。
    
    当前支持的控件类型如下：
    - code: 代码类型控件，支持语法高亮
    - text: 文本类型控件，不支持语法高亮
    - number: 数字类型
    - choices: 选项类型
    """,
    )

    @model_validator(mode="after")
    def check_valid(self, info: ValidationInfo) -> Self:
        context = info.context
        if context and context.get("strict"):
            if not self.desc:
                raise ValueError("ParamDef desc must be set")
            if not self.input_widget:
                raise ValueError("ParamDef inputWidget must be set")

        return self


class Entry(BaseModel):
    load: str
    run: str


class OsType(str, Enum):
    Linux = "linux"
    Windows = "windows"
    Darwin = "darwin"
    Android = "android"


class ArchType(str, Enum):
    Amd64 = "amd64"
    Arm64 = "arm64"


class TestTool(BaseModel):
    __test__ = False

    """
    测试工具模型定义
    """

    schema_version: float = Field(alias="schemaVersion")
    name: str = Field(
        pattern=r"^[a-zA-Z-0-9_]+$",
        title="工具名称",
        description="允许英文字母+数字+`-`",
    )
    legacy_name: str = Field(
        "",
        alias="legacyName",
        title="遗留工具名称",
        description="""
由于部分工具还未切换到TestSolar引擎，因此增加一个遗留工具字段。

- TestSolar工具此字段为空
- 遗留工具此字段为老的工具ID(比如pytest)
    """,
    )
    description: str = Field(
        min_length=10,
        max_length=1000,
        title="工具描述",
        description="""
工具描述信息，10~1000个字符。

简要描述工具的使用场景。
    """,
    )

    # x.x.x 格式版本
    version: str = Field(
        pattern=r"^(\d+\.\d+\.\d+|stable)$",
        title="工具版本",
        description="""
工具版本，需要符合一定格式：

- x.x.x: 满足语义化版本(https://semver.org/lang/zh-CN/)
- stable: 用于指定稳定版本，通常是最后一次发布的版本，以方便用户使用
    """,
    )
    lang: Literal["python", "golang", "javascript", "java", "cpp"]
    base_image: str = Field(
        alias="defaultBaseImage",
        title="使用的默认镜像",
        description="""
在没有指定镜像时，TestSolar会使用此默认镜像作为基础镜像。

用户也可以使用其他自定义镜像来满足自己的需求。
    """,
    )
    lang_type: Literal["COMPILED", "INTERPRETED"] = Field(
        alias="langType",
        title="语言类型",
        description="""
指定是解释型语言还是编译型语言。

- COMPILED: 编译型语言
- INTERPRETED: 解释型语言
    """,
    )
    param_defs: list[ParamDef] | None = Field(None, alias="parameterDefs")
    home_page: str = Field(alias="homePage", title="工具首页")
    version_file: str = Field(
        alias="versionFile",
        title="版本文件地址",
        description="""
版本文件地址，指向历史版本元数据文件。

版本文件中，保存了测试工具发布的历史版本。TestSolar测试工具的用户可以选择某个特定的历史版本使用。
    """,
    )
    index_file: str = Field(
        alias="indexFile",
        title="稳定版本索引",
        description="""
稳定版本索引，包含各个测试工具的稳定版本元数据信息。

由于稳定版本是通常使用的版本，因此外部系统仅需要访问此稳定版本元数据信息，即可快速读取需要使用的测试工具数据。
    """,
    )
    scaffold_repo: str = Field(
        "",
        alias="scaffoldRepo",
        title="脚手架仓库URL",
        description="""
指向测试工具使用的脚手架仓库信息。

TestSolar使用此信息来快速生成一个测试工具的用例库，用户可在此用例库的基础上继续开发。
    """,
    )
    support_os: list[OsType] | None = Field(None, alias="supportOS", title="支持的系统")
    support_arch: list[ArchType] | None = Field(
        None, alias="supportArch", title="支持的CPU架构"
    )
    entry: Entry | None = Field(
        None,
        alias="entry",
        title="测试工具交互入口文件",
        description="""
定义跟uniSDK之间交互的调用方式。

使用场景：
- 测试工具install.sh/install.ps脚本中需要安装uniSDK
- 测试工具run.sh/run.ps脚本中需要启动uniSDK并指定使用的测试工具

entry中需要定义2个入口：
- 加载用例入口(load)
- 执行用例入口(run)

实际执行的时候会把字符串中定义的 $1 换成入口参数文件。

具体使用方式请参考uniSDK的相关文档。
    """,
    )
    git_pkg_url: str = Field(
        "",
        alias="gitPkgUrl",
        title="Git方式使用地址",
        description="""
指定测试工具在Git模式下的使用地址。

通常不推荐，我们优先推荐使用Http元数据方式。
- Http方式无需安装Git
- Http方式无需Git认证，下载成功率更高
    """,
    )
    name_zh: str = Field("", alias="nameZh", title="工具中文名称")
    legacy_spec: LegacySpec | None = Field(
        None,
        alias="legacySpec",
        title="遗留工具规范",
        description="""
    老的遗留工具中使用的一些额外字段。
    
    这些字段仅在遗留工具中使用，TestSolar平台工具不再支持。
    """,
    )
    certified: bool = Field(
        False,
        alias="certified",
        title="是否是TestSolar官方认证插件",
        description="""
是否是TestSolar官方支持工具（包括遗留工具）。

官方支持工具由TestSolar官方维护，质量上有保证，能够得到官方支持。
        """,
    )

    archived: bool = Field(
        False,
        alias="archived",
        title="是否已经归档",
        description="""
部分测试工具已经无人使用，标记为归档。

各系统可根据此信息来决定相关处理策略。        
        """
    )

    @model_validator(mode="after")
    def check_valid(self, info: ValidationInfo) -> Self:
        """
        检查测试工具定义是否合法

        直接在模型中增加非None检查会导致旧版本的测试工具元数据解析报错，所以单独提取一个函数用于校验，需要的时候再调用
        """
        context = info.context
        if context and context.get("strict"):
            if not self.support_os:
                raise ValueError("supportOS must be set")
            if not len(self.support_os) > 0:
                raise ValueError("need at least 1 support OS")

            if not self.support_arch:
                raise ValueError("need at least 1 support arch")
            if not len(self.support_arch) > 0:
                raise ValueError("need at least 1 support arch")

            if not self.legacy_spec:
                if not self.git_pkg_url:
                    raise ValueError("gitPkgUrl must be set")
                if not self.version_file:
                    raise ValueError("versionFile must be set")
                if not self.index_file:
                    raise ValueError("indexFile must be set")
                if not self.scaffold_repo:
                    raise ValueError("scaffoldRepo must be set")
            else:
                if not self.legacy_name:
                    raise ValueError("legacyName must be set")

            if not self.name_zh:
                raise ValueError("name_zh must be set")

        return self


class TestToolTarget(BaseModel):
    __test__ = False

    """
    发布包模型定义
    """

    os: OsType
    arch: ArchType
    download_url: str = Field(alias="downloadUrl")
    sha256: str


class StableIndexMetaData(BaseModel):
    """
    稳定版本索引文件
    """

    meta_version: str = Field("1", alias="metaVersion")
    tools: list[TestTool]

    def merge_stable_index(self, tools_want_to_merge: list[TestTool]) -> None:

        if not self.tools:
            self.tools = []

        for tool_to_merge in tools_want_to_merge:
            for index, tool in enumerate(self.tools):
                if tool.name == tool_to_merge.name:
                    self.tools[index] = tool_to_merge
                    break
            else:
                self.tools.append(tool_to_merge)


class TestToolMetadata(BaseModel):
    __test__ = False

    """
    通过solar-registry生成的最新版本发布元数据

    包含元数据信息和target信息
    """

    meta: TestTool
    target: list[TestToolTarget]


class MetaDataHistory(BaseModel):
    """
    工具元数据版本文件
    """

    meta_version: str = Field("1", alias="metaVersion")
    versions: list[TestToolMetadata]



import tempfile
from pathlib import Path
from typing import Callable

from solar_registry.commands.meta_merger import MetaMerger
from solar_registry.model.test_tool import MetaDataHistory, TestToolMetadata
from solar_registry.service.testtool import get_testtool, github_asset_gen


def test_merge_meta_file() -> None:
    with tempfile.TemporaryDirectory() as tmpdir:
        workdir = str(Path(__file__).parent.joinpath("testdata").resolve())
        testtool = get_testtool(tool_name="pytest", workdir=workdir)
        gen = MetaMerger(testtool, asset_url_gen=github_asset_gen)
        gen.merge_index_and_history(Path(tmpdir))

        index_file = Path(tmpdir) / "testtools/stable.index.json"
        meta_file = Path(tmpdir) / "testtools/python/pytest/metadata.json"

        assert index_file.exists()
        assert meta_file.exists()

        with open(meta_file) as f:
            history = MetaDataHistory.model_validate_json(f.read())

            # 能够找到stable版本和当前版本，并且数据一致
            stable = find_if(history.versions, lambda x: x.meta.version == "stable")
            assert stable

            current = find_if(
                history.versions, lambda x: x.meta.version == testtool.version
            )
            assert current

            assert stable.target == current.target


def find_if(lst: list[TestToolMetadata], predicate: Callable[[TestToolMetadata], bool]) -> TestToolMetadata | None:
    return next((x for x in lst if predicate(x)), None)



import os
from pathlib import Path

import yaml
from loguru import logger

from ..model.test_tool import TestTool


def get_testtool(tool_name: str, workdir: str | None) -> TestTool:
    logger.debug(f"querying testtool for {tool_name}")
    workdir = workdir or os.getcwd()
    return _parse_testtool(Path(workdir) / tool_name / "testtool.yaml", strict=True)


def _parse_testtool(yaml_file: Path, strict: bool) -> TestTool:
    with open(yaml_file) as f:
        context = {}
        if strict:
            context["strict"] = True

        testtool = TestTool.model_validate(yaml.safe_load(f), context=context)
        logger.debug(
            f"loaded testtool: {testtool.model_dump_json(by_alias=True, indent=2, exclude_none=True)}"
        )
        return testtool


def github_asset_gen(testtool: TestTool) -> str:
    return f"https://github.com/OpenTestSolar/testtool-{testtool.lang}-{testtool.name}/archive/refs/tags/{testtool.version}.tar.gz"



import hashlib
import os
import tarfile
import tempfile
from pathlib import Path
from typing import Callable

from loguru import logger
from qcloud_cos import CosS3Client, CosConfig  # type: ignore[import-untyped]

from ..model.asset import gen_asset_relative_path
from ..model.test_tool import TestTool


class CosSyncService:
    def __init__(
        self,
        workdir: str | None = None,
        endpoint_gen: Callable[[str], str] | None = None,
    ) -> None:
        self.workdir = workdir or os.getcwd()
        if "COS_SECRET_ID" not in os.environ:
            raise ValueError("Environment COS_SECRET_ID variables not set")
        self.cos_secret_id = os.environ["COS_SECRET_ID"]
        if "COS_SECRET_KEY" not in os.environ:
            raise ValueError("Environment COS_SECRET_KEY variables not set")
        self.cos_secret_key = os.environ["COS_SECRET_KEY"]
        if "COS_REGION" not in os.environ:
            raise ValueError("Environment COS_REGION variables not set")
        self.cos_region = os.environ["COS_REGION"]
        if "COS_BUCKET" not in os.environ:
            raise ValueError("Environment COS_BUCKET variables not set")
        self.cos_bucket = os.environ["COS_BUCKET"]

        if endpoint_gen:
            config = CosConfig(
                Region=self.cos_region,
                SecretId=self.cos_secret_id,
                SecretKey=self.cos_secret_key,
                Endpoint=endpoint_gen(self.cos_region),
            )
        else:
            config = CosConfig(
                Region=self.cos_region,
                SecretId=self.cos_secret_id,
                SecretKey=self.cos_secret_key,
            )

        self.cos_client = CosS3Client(config)

    def sync_meta_data(self, force: bool) -> None:
        for dir_path, _, filenames in os.walk(Path(self.workdir) / "testtools"):
            for filename in filenames:
                full_path = Path(dir_path, filename)
                logger.info(f"Syncing {full_path}")
                relative_path = os.path.relpath(full_path, self.workdir)
                logger.info(f"Relative path = {relative_path}")

                # 上传本地文件，相同文件跳过不上传
                self.upload_relative_file(relative_path, force)

    def upload_relative_file(self, relative_file: str, force: bool) -> None:
        full_path = Path(self.workdir, relative_file)

        if force:
            logger.info(f"Overwriting {relative_file}...")
            self.upload_file_to_cos(
                relative_file=relative_file, full_path=str(full_path)
            )
        else:
            if self.cos_client.object_exists(Bucket=self.cos_bucket, Key=relative_file):
                with tempfile.TemporaryDirectory() as temp:
                    output_path = Path(temp) / relative_file
                    output_path.parent.mkdir(parents=True, exist_ok=True)

                    logger.info(f"Download {relative_file} to {output_path}...")
                    self.cos_client.download_file(
                        Bucket=self.cos_bucket,
                        Key=relative_file,
                        DestFilePath=str(output_path),
                    )

                    logger.info("Compare MD5...")
                    if calculate_md5(full_path) == calculate_md5(output_path):
                        logger.info(
                            f"✨ relative_file {relative_file} not changed, skip upload"
                        )
                    else:
                        self.upload_file_to_cos(
                            relative_file=relative_file, full_path=str(full_path)
                        )
            else:
                logger.info(
                    f"File {relative_file} does not exist on cos, start upload..."
                )
                self.upload_file_to_cos(
                    relative_file=relative_file, full_path=str(full_path)
                )

    def upload_archive_file(self, test_tool: TestTool) -> None:
        # 生成对应测试工具的压缩包，并上传到COS的指定位置
        with tempfile.TemporaryDirectory() as tmpdir:
            output_file = Path(tmpdir) / f"{test_tool.name}.tar.gz"
            with tarfile.open(output_file, "w:gz") as tar:
                tar.add(Path(self.workdir) / test_tool.name, arcname=test_tool.name)

            cos_file_path = gen_asset_relative_path(testtool=test_tool)
            logger.info(f"Uploading {cos_file_path} to COS bucket {self.cos_bucket}...")
            self.upload_file_to_cos(
                relative_file=cos_file_path, full_path=str(output_file)
            )
            logger.info(f"✅ Uploaded {cos_file_path} to COS bucket {self.cos_bucket}")

    def upload_file_to_cos(self, relative_file: str, full_path: str) -> None:
        response = self.cos_client.upload_file(
            Bucket=self.cos_bucket,
            Key=relative_file,
            LocalFilePath=full_path,
            EnableMD5=True,
            progress_callback=None,
        )
        logger.info(
            f"✅ relative_file {relative_file} uploaded, ETag: {response['ETag']}"
        )


def calculate_md5(file_path: Path) -> str:
    """
    计算文件的 MD5 码。

    Args:
        file_path: 文件路径。

    Returns:
        文件的 MD5 码。
    """

    with open(file_path, "rb") as f:
        file_content = f.read()
        md5_hash = hashlib.md5()
        md5_hash.update(file_content)
        return md5_hash.hexdigest()



from typing import Literal

from pydantic import BaseModel, Field


class MetaExtraField(BaseModel):
    cli: str | None = None


class LegacySpec(BaseModel):
    testcase_runner: MetaExtraField = Field(alias="testcaseRunner")
    testcase_loader: MetaExtraField = Field(alias="testcaseLoader")
    testcase_analyzer: MetaExtraField = Field(alias="testcaseAnalyzer")
    scaffolding_tool: MetaExtraField = Field(alias="scaffoldingTool")
    node_setup: MetaExtraField = Field(alias="nodeSetup")
    node_cleanup: MetaExtraField = Field(alias="nodeCleanup")
    global_setup: MetaExtraField = Field(alias="globalSetup")
    global_cleanup: MetaExtraField = Field(alias="globalCleanup")
    report_type: Literal["qta", "junit"] = Field(alias="reportType")
    res_pkg_url: str = Field(alias="resPkgUrl")
    doc_url: str = Field(alias="docUrl")
    logo_img_url: str = Field(alias="logoImgUrl")
    enable_code_coverage: bool = Field(alias="enableCodeCoverage")
    maintainers: list[str] = Field(alias="maintainers")



from pathlib import Path

import pytest
from pydantic import ValidationError

from solar_registry.service.testtool import get_testtool, _parse_testtool
from solar_registry.model.test_tool import OsType, ArchType, ParamWidget


def test_validate_correct_pytest_tool() -> None:
    workdir = str((Path(__file__).parent / "testdata").resolve())

    tool = get_testtool("pytest", workdir)

    assert tool.name == "pytest"
    assert tool.version == "0.1.6"
    assert tool.name_zh == "pytest自动化测试"
    assert tool.git_pkg_url == "github.com/OpenTestSolar/testtool-python@main:pytest"

    assert tool.support_os
    assert tool.support_os[0] == OsType.Windows
    assert tool.support_os[1] == OsType.Linux
    assert tool.support_os[2] == OsType.Darwin

    assert tool.support_arch
    assert tool.support_arch[0] == ArchType.Amd64
    assert tool.support_arch[1] == ArchType.Arm64


def test_validate_loose() -> None:
    _parse_testtool(Path(__file__).parent / "testdata" / "error_meta_files" / "error_yaml_with_loose.yaml", strict=False)


def test_validate_strict_with_errors() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(Path(__file__).parent / "testdata" / "error_meta_files" / "error_yaml_with_strict.yaml", strict=True)

    print(ve.value)

    assert r"String should match pattern '^[a-zA-Z-0-9_]+$'" in str(ve.value)
    assert r"Input should be 'COMPILED' or 'INTERPRETED'" in str(ve.value)
    assert r"Input should be 'python', 'golang', 'javascript', 'java' or 'cpp'" in str(ve.value)
    assert r"String should match pattern '^(\d+\.\d+\.\d+|stable)$'" in str(ve.value)
    assert r"Input should be 'linux', 'windows', 'darwin' or 'android'" in str(ve.value)
    assert r"Input should be 'amd64' or 'arm64'" in str(ve.value)


def test_validate_strict_with_no_os() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(Path(__file__).parent / "testdata" / "error_meta_files" / "error_yaml_with_strict_no_os.yaml", strict=True)

    print(ve.value)
    assert r"Value error, supportOS must be set" in str(ve.value)


def test_validate_strict_with_no_arch() -> None:
    with pytest.raises(ValidationError) as ve:
        _parse_testtool(Path(__file__).parent / "testdata" / "error_meta_files" / "error_yaml_with_strict_no_arch.yaml", strict=True)

    print(ve.value)
    assert r" Value error, need at least 1 support arch" in str(ve.value)


def test_parse_legacy_tool() -> None:
    tool = _parse_testtool(Path(__file__).parent / "testdata" / "legacy" / "testtool.yaml", strict=True)

    assert tool.name == "qt4s_pot-line"
    assert not tool.git_pkg_url
    assert tool.param_defs
    param1 = tool.param_defs[0]
    assert param1.name == "setup_cmdline"
    assert param1.value == "初始化命令"
    assert param1.desc == "加载/执行用例前的初始化命令"
    assert param1.default == "# place your extra init command here"
    assert param1.lang == "bash"
    assert param1.input_widget == ParamWidget.Code

    assert tool.legacy_spec
    assert tool.legacy_name == "qt4s"
    assert tool.legacy_spec.report_type == "junit"
    assert tool.legacy_spec.maintainers == ["aa", "bb"]
    assert tool.legacy_spec.testcase_runner.cli == "test"
    assert tool.legacy_spec.testcase_loader.cli == "test"
    assert tool.legacy_spec.testcase_analyzer.cli == "test"
    assert tool.legacy_spec.scaffolding_tool.cli == "test"
    assert tool.legacy_spec.node_setup.cli == "test"
    assert tool.legacy_spec.node_cleanup.cli == "test"
    assert tool.legacy_spec.global_setup.cli == "test"
    assert tool.legacy_spec.global_cleanup.cli == "test"
    assert tool.legacy_spec.res_pkg_url == "https://sample.com/pkg"
    assert tool.legacy_spec.doc_url == "https://sample.com/doc"
    assert tool.legacy_spec.logo_img_url == "https://sample.com/logo"
    assert not tool.legacy_spec.enable_code_coverage
